/*   viewer3d.h
* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*            National Center for Biotechnology Information (NCBI)
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government do not place any restriction on its use or reproduction.
*  We would, however, appreciate having the NCBI and the author cited in
*  any work or product based on this material
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
* ===========================================================================
*
* File Name:  viewer3d.h
*
* Author:  Alex Smirnov
*
* Version Creation Date:   03/29/95
*
* $Revision: 5.2 $
*
* File Description: 
*
* Modifications:  
* --------------------------------------------------------------------------
* Date     Name        Description of modification
* -------  ----------  -----------------------------------------------------
*
* 29 Mar 96  C. Hogue   V1.4 Added log line below:
* 28 Mar 96  C. Hogue   V1.3 Added Layer Controls
* ==========================================================================
* $Log: viewer3d.h,v $
 * Revision 5.2  1996/06/14  16:39:21  kans
 * copy and print 3d viewer now ifndef win_motif
 *
 * Revision 5.1  1996/06/14  14:29:58  vakatov
 * Nlm_SetBackground3D() added to change 3D-Viewer background color.
 * "Copy" and "Print" buttons related stuff carefully commented everywhere.
 *
 * Revision 5.0  1996/05/28  13:45:08  ostell
 * Set to revision 5.0
 *
 * Revision 1.13  1996/05/23  14:36:35  hogue
 * Fixed PrintViewer/PasteViewer discrepancy in .h file, added
 * IsPlaying3D, StopPlaying3D, StartPlaying3D for more
 * external control of the animation
 *
 * Revision 1.12  1996/05/13  15:04:38  hogue
 * Save button in Viewer Controls now optional
 *
 * Revision 1.11  1996/05/09  18:30:38  vakatov
 * Added CAMERA_SIZE_I4 constant for an external use
 *
 * Revision 1.10  1996/05/07  18:30:25  vakatov
 * Nlm_CreateViewer3D() prototype -- modified
 *
 * Revision 1.9  1996/04/17  21:31:33  vakatov
 * Added "Nlm_GetViewerImage3D()" function returning image of the 3D-viewer.
 *
 * Revision 1.8  1996/04/07  20:04:56  kans
 * getlayertop3d needed (void) as prototype
 *
 * Revision 1.7  1996/04/04  18:52:45  vakatov
 * The camera position storing structure size is increased
 *
 * Revision 1.6  1996/04/02  22:30:53  vakatov
 * AttachZoomPic3D() removed; now previously stored camera position
 * is to be restored within AttachPicture3D automatically, as soon as
 * valid (non-NULL and initialized) "camera" argument is provided
 *
 * Revision 1.5  1996/04/01  23:03:09  vakatov
 * Added "dscale" member to the "Nlm_camera3d" structure
 *
 * Revision 1.4  1996/03/30  23:25:55  hogue
 * Added AttachZoomPic3D
 *
*
*/

#ifndef _VIEWER3D_
#define _VIEWER3D_

#ifndef _VIBRANT_
#include <vibrant.h>
#endif

#ifndef _PICTUR3D_
#include <pictur3d.h>
#endif

#include <image.h>

#ifdef __cplusplus
extern "C" {
#endif

/*****************************************************************************
*
*   DEFINES
*
*****************************************************************************/
/* Types of adjust for Nlm_SetAdjust3D() */
#define ADJUST3D_PERSPECT   1
#define ADJUST3D_BRIGH      2
#define ADJUST3D_COLOR      3

/*****************************************************************************
*
*   TYPEDEFS
*
*****************************************************************************/
typedef struct Nlm_viewer3d {
  VoidPtr  dummy;
} PNTR Nlm_Viewer3D;

#define CAMERA_SIZE_I4 17
typedef struct Nlm_camera3d {
  Int4  dummy[CAMERA_SIZE_I4];
} Nlm_Camera3D, PNTR Nlm_Camera3DPtr;

typedef void (*Nlm_V3DClckProc) PROTO((Nlm_Viewer3D, PoinT));

typedef struct Nlm_v3dcontrols {
  ButtoN  rotate;
  ButtoN  zoomAll;
  ButtoN  zoomPan;
  ButtoN  zoomWin;
  GrouP   adjustButtons;
  ButtoN  plusButton;
  ButtoN  minusButton;
  ButtoN  saveButton;
/* layer - animation controls */
  ButtoN  allButton;
  ButtoN  rewindButton;
  ButtoN  prevButton;
  ButtoN  nextButton;
  ButtoN  playButton;
#ifndef WIN_MOTIF
/*
  ButtoN  copyButton;
  ButtoN  printButton;
*/
#endif
} Nlm_V3DControls, PNTR Nlm_V3DControlsPtr;

/*****************************************************************************
*
*   EXTERN FUNCTION
*
*****************************************************************************/
extern Nlm_Viewer3D  Nlm_CreateViewer3D    PROTO((Nlm_GrouP prnt, 
                                    Uint2Ptr width, Uint2 height, 
                                    Boolean vscroll, Boolean hscroll)); 
extern GrouP         Nlm_CreateControl3D   PROTO((Nlm_GrouP prnt, 
                                    Boolean vertical, 
                                    Nlm_V3DControlsPtr controls));
extern GrouP         Nlm_CreateControl3DS   PROTO((Nlm_GrouP prnt, 
                                    Boolean vertical, Boolean savebtn,
                                    Nlm_V3DControlsPtr controls));
extern Pointer       Nlm_GetViewerExtra3D  PROTO((Nlm_Viewer3D viewer));
extern Image         Nlm_GetViewerImage3D  PROTO((Nlm_Viewer3D viewer));
extern void          Nlm_LinkControls3D    PROTO((Nlm_V3DControlsPtr controls,
                                    Pointer extra));
extern void          Nlm_DeleteViewer3D    PROTO((Nlm_Viewer3D viewer));
extern Boolean       Nlm_AttachPicture3D   PROTO((Nlm_Viewer3D viewer, 
                                    Picture3D picture, 
                                    Nlm_Camera3DPtr camera));
extern void          Nlm_ResetViewer3D     PROTO((Nlm_Viewer3D viewer));
extern void          Nlm_GetViewerInfo3D   PROTO((Nlm_Viewer3D viewer,
                                    Picture3D PNTR pic,
                                    Nlm_Camera3DPtr camera,
                                    RectPtr rectangle));
extern Boolean       Nlm_RedrawViewer3D    PROTO((Nlm_Viewer3D viewer));
extern void          Nlm_SetViewer3DProcs  PROTO((Nlm_Viewer3D viewer, 
                                    Nlm_V3DClckProc click,
                                    Nlm_V3DClckProc drag,
                                    Nlm_V3DClckProc release));
extern Uint2         Nlm_FindPrim3D        PROTO((Nlm_Viewer3D viewer, 
                                    PoinT pt));
extern Prim3D        Nlm_GetFoundPrim3D    PROTO((Nlm_Viewer3D viewer, 
                                    Uint2 primIndex));
extern void          Nlm_SetHLColor3D      PROTO((Nlm_Viewer3D viewer,
                                    Uint1 red, Uint1 green, Uint1 blue));
extern void          Nlm_HighlightSeg3D    PROTO((Nlm_Viewer3D viewer,
                                    Segment3D segment, Boolean highlight));
extern Boolean       Nlm_IsSeg3DHlighted   PROTO((Nlm_Viewer3D viewer, 
                                    Segment3D segment));
extern void          Nlm_HighlightPrim3D   PROTO((Nlm_Viewer3D viewer,
                                    Prim3D prim, Boolean highlight));
extern Boolean       Nlm_IsPrim3DHlighted  PROTO((Nlm_Viewer3D viewer, 
                                    Prim3D prim));
extern Boolean       Nlm_ResetHighlight3D  PROTO((Nlm_Viewer3D viewer));
extern Boolean       Nlm_MapWorldToViewer3D  PROTO((Nlm_Viewer3D viewer,
                                    Int4 x, Int4 y, Int4 z,
                                    PoinT PNTR vPoint));


extern void          Nlm_SetBackground3D   PROTO ( (Nlm_Viewer3D viewer,
                                                    Nlm_Uint1 red,
                                                    Nlm_Uint1 green,
                                                    Nlm_Uint1 blue) );

/* Functions, that can be used im Button callbacks */
extern void          Nlm_SetQRotate3D      PROTO((Pointer v3Dextra, 
                                    Boolean onoff));
extern Boolean       Nlm_GetQRotate3D	   PROTO(( Pointer v3Dextra));
extern void          Nlm_ZoomAll3D         PROTO((Pointer v3Dextra)); 
extern void          Nlm_ZoomPan3D         PROTO((Pointer v3Dextra)); 
extern void          Nlm_ZoomWin3D         PROTO((Pointer v3Dextra)); 
extern void          Nlm_SetAdjust3D       PROTO((Pointer v3Dextra,
                                    Int2 ajast)); 
extern void          Nlm_AdjustPlus3D      PROTO((Pointer v3Dextra));
extern void          Nlm_AdjustMinus3D     PROTO((Pointer v3Dextra));
extern void          Nlm_SaveViewer3D      PROTO((Pointer v3Dextra));
#ifndef WIN_MOTIF
extern void          Nlm_CopyViewer3D      PROTO((Pointer v3Dextra));
extern void          Nlm_PrintViewer3D     PROTO((Pointer v3Dextra));
#endif
extern void 	     Nlm_LinkAnimCtrls3D       PROTO((Nlm_V3DControlsPtr controls, Pointer extra ));
extern GrouP         Nlm_CreateAnimControl3D   PROTO((Nlm_GrouP prnt, 
                                    Boolean vertical, Boolean savebtn,
                                    Nlm_V3DControlsPtr controls));
extern void 	     Nlm_PlayLayer3D      PROTO(( Pointer v3Dextra ));
extern void 	     Nlm_NextLayer3D      PROTO(( Pointer v3Dextra ));
extern void    	     Nlm_PrevLayer3D      PROTO(( Pointer v3Dextra ));
extern void	     Nlm_SetLayerTop3D    PROTO(( Uint1 top));
extern Uint1         Nlm_GetLayerTop3D    PROTO((void));
extern Boolean       Nlm_IsPlaying3D      PROTO((void));
extern void          Nlm_StopPlaying3D    PROTO((void));
extern void          Nlm_StartPlaying3D    PROTO(( Pointer v3Dextra));

/*****************************************************************************
*
*   NAME REDIFINITION
*
*****************************************************************************/
#define Viewer3D            Nlm_Viewer3D
#define Camera3D            Nlm_Camera3D
#define Camera3DPtr         Nlm_Camera3DPtr
#define V3DClckProc         Nlm_V3DClckProc
#define V3DControlsPtr      Nlm_V3DControlsPtr
#define V3DControlsPtr      Nlm_V3DControlsPtr
#define CreateViewer3D      Nlm_CreateViewer3D
#define CreateControl3D     Nlm_CreateControl3D
#define CreateControl3DS    Nlm_CreateControl3DS
#define GetViewerExtra3D    Nlm_GetViewerExtra3D
#define GetViewerImage3D    Nlm_GetViewerImage3D
#define LinkControls3D      Nlm_LinkControls3D
#define DeleteViewer3D      Nlm_DeleteViewer3D
#define AttachPicture3D     Nlm_AttachPicture3D
#define ResetViewer3D       Nlm_ResetViewer3D
#define GetViewerInfo3D     Nlm_GetViewerInfo3D
#define RedrawViewer3D      Nlm_RedrawViewer3D
#define SetViewer3DProcs    Nlm_SetViewer3DProcs
#define FindPrim3D          Nlm_FindPrim3D
#define GetFoundPrim3D      Nlm_GetFoundPrim3D
#define SetHLColor3D        Nlm_SetHLColor3D
#define HighlightSeg3D      Nlm_HighlightSeg3D
#define IsSeg3DHlighted     Nlm_IsSeg3DHlighted
#define HighlightPrim3D     Nlm_HighlightPrim3D
#define IsPrim3DHlighted    Nlm_IsPrim3DHlighted
#define ResetHighlight3D    Nlm_ResetHighlight3D
#define MapWorldToViewer3D  Nlm_MapWorldToViewer3D
#define SetBackground3D     Nlm_SetBackground3D
#define SetQRotate3D        Nlm_SetQRotate3D
#define GetQRotate3D	    Nlm_GetQRotate3D
#define ZoomAll3D           Nlm_ZoomAll3D
#define ZoomPan3D           Nlm_ZoomPan3D
#define ZoomWin3D           Nlm_ZoomWin3D
#define ZoomWin3D           Nlm_ZoomWin3D
#define SetAdjust3D         Nlm_SetAdjust3D
#define AdjustPlus3D        Nlm_AdjustPlus3D
#define AdjustMinus3D       Nlm_AdjustMinus3D
#define SaveViewer3D        Nlm_SaveViewer3D
#ifndef WIN_MOTIF
#define CopyViewer3D        Nlm_CopyViewer3D
#define PrintViewer3D       Nlm_PrintViewer3D
#endif
#define LinkAnimCtrls3D     Nlm_LinkAnimCtrls3D
#define CreateAnimControl3D Nlm_CreateAnimControl3D
#define PlayLayer3D	    Nlm_PlayLayer3D   
#define NextLayer3D         Nlm_NextLayer3D      
#define PrevLayer3D   	    Nlm_PrevLayer3D     
#define SetLayerTop3D       Nlm_SetLayerTop3D
#define GetLayerTop3D       Nlm_GetLayerTop3D 
#define IsPlaying3D         Nlm_IsPlaying3D
#define StopPlaying3D       Nlm_StopPlaying3D
#define StartPlaying3D      Nlm_StartPlaying3D


#ifdef __cplusplus
}
#endif

#endif

